#! /usr/bin/python
import sys, getopt, os, string, gdbm, struct, commands
programname = 'pimp3'
versionnumber = '0.5.8'
#insert the pathname of the folder where you store your mp3s
mp3folder = ' '
databasefile = os.environ['HOME'] + "/." + programname + ".db"
configfile = os.environ['HOME'] + "/." + programname + ".conf"

#see if the db file exists(needed in database.rebuild)
notfirsttime = 1 #used to see if this is a virgin database
command = os.environ['HOME'] + "/." + programname + ".db"
status = commands.getstatus(command)
status = string.split(status, "No")
if(len(status) > 1):
    notfirsttime = 0
try: 
    db = gdbm.open(databasefile, 'w')
except: 
    db = gdbm.open(databasefile, 'c')
try:
    del db['halt']
    db['halt'] = struct.pack('i', 1)
except:
    db['halt'] = struct.pack('i', 1)
try: 
    conf = open(configfile, 'r')
    mp3folder = conf.readline()
    mp3folder = string.strip(mp3folder)
except: 
    conf = open(configfile, 'w')
    mp3folder = raw_input('where do you store your mp3s? ')
    x = string.split(mp3folder, '/')
    count = 1
    y =  '/'+ x[0]
    while count < len(x):
	y = y + x[count] + '/'
	count = count + 1
    mp3folder = y
    print mp3folder
    del x, y, count
    conf.write(mp3folder)
    
def help():
        "print all the command line options"
	print "Usage: ", programname, " [OPTIONS]"
	print "A smart mp3 player"
	print
	print "--help        -h   print this help screen"
	print "--version     -v   print the program version"
	print "--play        -p   play the music"
	print "--rebuild     -r   rebuild the music database"
	print "--distribute  -d   rebuild the database and"
	print "                   distribute the ratings(first time only)"
	print "--setup       -s   setup the music database(rate the songs)"
	print "--make_setup  -m   make the setup file for setup"
	print "--print_amout -c   print the amount of mp3s in each category"
	print "                   this is useful to balance the load of mp3s"
	print "                   in each category. You want more in the 5"
	print "                   category than the 1 category."
	print "-two          -2   play only songs of the value 2 and above"
	print "-three        -3   play only songs of the value 3 and above"
	print "-four         -2   play only songs of the value 4 and above"
	print "-five         -3   play only songs of the value 5"
	sys.exit()
						    
def version():
    "print the version number"
    print programname, versionnumber
    sys.exit()		
	
try: optlist, args = getopt.getopt(sys.argv[1:], "lcrhvspmd2345", ["rebuild", "help",
						    "version", "setup",
						    "print_all", "play",
						"list","make_setup", "print_amount", "distribute"
						"two", "three", "four", "five"])
except:
    help()
for i in optlist:
    if i[0] in ["--rebuild", "-r"]:
	import database
	database.rebuild(db, mp3folder, programname, notfirsttime, spreadratings = 0)
    elif i[0] in ["--distribute", "-d"]:
	import database
	database.rebuild(db, mp3folder, programname, notfirsttime, spreadratings = 1)
    elif i[0] in ["--setup", "-s"]:
	import database
	database.setup(db, programname)
    elif i[0] in ["--make_setup", "-m"]:
	import database
	database.makesetup(db, mp3folder, programname)
    elif i[0] in ["--list", "-l"]:
	import database
	database.getlist(mp3folder)
    elif i[0] in ["--print_amount", "-c"]:
	import play
	play.print_amount(db)
    elif i[0] in ["--play", "-p"]:
	import play
	play.randomplay(db, 1, versionnumber)
    elif i[0] in ["--two", "-2"]:
	import play
	play.randomplay(db, 2, versionnumber)
    elif i[0] in ["--three", "-3"]:
	import play
	play.randomplay(db, 3, versionnumber)
    elif i[0] in ["--four", "-4"]:
	import play
	play.randomplay(db, 4, versionnumber)
    elif i[0] in ["--five", "-5"]:
	import play
	play.randomplay(db, 5, versionnumber)
    elif i[0] in ["--help", "-h"]:
	help()
    elif i[0] in ["--version", "-v"]:
	version()

if args != []:
    help()



import pickle, database, string, gdbm, sys, whrandom 
import struct, commands, thread, signal, os, FCNTL
def print_amount(db):
    "print the sizes of the songs"
    one = []             #make the data structures to store the numbers in
    two = []
    three = []
    four = []
    five = []
    six = []
    seven = []
    eight = []
    nine = []
    ten = []
    k = db.firstkey()
    while k != None:
	value = db[k]                   #get each song's rating
	songname = string.lstrip(k)     #from the database
	if value == '1':
	    one.append(songname)
	elif value == '2':
	    two.append(songname)
	elif value == '3':
	    three.append(songname)
	elif value == '4':
	    four.append(songname)
	elif value == '5':
	    five.append(songname)
	elif value == '6':
	    six.append(songname)
	elif value == '7':
	    seven.append(songname)
	elif value == '8':
	    eight.append(songname)
	elif value == '9':
	    nine.append(songname)
	elif value == '10':
	    ten.append(songname)
	k = db.nextkey(k)
    
    print "1:", len(one)
    print "2:", len(two)
    print "3:", len(three)
    print "4:", len(four)
    print "5:", len(five)
    print "6:", len(six)
    print "7:", len(seven)
    print "8:", len(eight)
    print "9:", len(nine)
    print "10:", len(ten)
    print "total:", len(one) + len(two) + len(three) + len(four) + len(five) + len(six) + len(seven) + len(eight) + len(nine) + len(ten)
def change_value(db, value):
    "change the value of the song that is currently playing"
    name = db['currentsong']
    if(db.has_key(name)):
	del db[name]
	value = database.int_to_string(value)
	db[name] = value
def next(db, playfrom):
    if(db.has_key('halt')):
	del db['halt']
	db['halt'] = 'a'
    else:
	db['halt'] = 'a'
    command = 'killall -9 mpg123'            #kill mpg123
    output = commands.getstatusoutput(command)
    name = db['currentsong']
    print 'songvalue reduced by 1'
    songvalue = db[name]                    #skip to the next song and decrese the song's rating
    del db[name]
    songvalue = database.subfromstr(songvalue)
    db[name] = songvalue
    #prevents next from exiting from next before play is dead.
    while 1:
	if db['playlock'] == 'locked':
	    pass
	else:
	    break
    thread.start_new_thread(playsongs, (db, playfrom, 0))

def prev(db, playfrom):
    try:
	del db['halt']
	db['halt'] = 'a'
    except:
	db['halt'] = 'a'
    command = 'killall -9 mpg123'            #kill mpg123
    output = commands.getstatusoutput(command)
    if(db.has_key('prevsong')):
	name = db['prevsong']
	print 'songvalue raised by 1'
	songvalue = db[name]                    #skip to the next song and decrese the song's rating
	del db[name]
	songvalue = database.addtostr(songvalue)
	db[name] = songvalue
    #prevents prev from exiting from prev before play is dead.
    while 1:
	if db['playlock'] == 'locked':
	    pass
	else:
	    break
    thread.start_new_thread(playsongs, (db, playfrom, 1))
    
def end_thread(db):
    "this prevents the program from looping in it's thread"
    #this uses the halt key to kill a playsongs() loop in progress
    #this is nescessary for skipping or playing a previous song
    x = 0
    try:
	haltvalue = db['halt']
	haltvalue = string.strip(haltvalue)
	del db['halt']
	if haltvalue == 'a':
	    db['halt'] = 'b'
	    x = 1
	else:
	    db['halt'] = 'b'
    except:
	print 'halt key not in db'
	db['halt'] = 'b'
	x = 0
    return x

def playsongs(db, playfrom, play_prev):
    #this is my semi-random song player thingy
    #get a random integer between 1 and 5, play a song of that value,
    #then subtract one from that values place holder, so that there are
    #fives songs of the value 5 played, 4 of four, etc.
    
    if(db.has_key('playlock')):       #lock so that no other play functions can be started
	if db['playlock'] == 'locked':
	    print 'crimeny it\'s locked'
	    thread.exit()
	else:
	    del db['playlock']
	    db['playlock'] = 'locked'
    else:
	db['playlock'] = 'locked'
    if(db.has_key('playvalues') == 0):
	count = 1
	playvalues = {}
	while count <= 10:
	    playvalues[count] = count
	    count = count + 1
	valuesfile= os.environ['HOME'] + "/.pimp3.values"
	f = open(valuesfile, 'w')
	p = pickle.Pickler(f)
	p.dump(playvalues)
	del f
	db['playvalues'] = 'yes'
    playto = 10   #initialize playto
    while 1:
	k = db.firstkey()
	one = []             #make the data structures to store the numbers in
	two = []
	three = []
	four = []
	five = []
	six = []
	seven = []
	eight = []
	nine = []
	ten = []
	while k != None:
	    value = db[k]                   #get each song's rating
	    songname = k     #from the database
	    if ((songname == 'halt')|(songname == 'song playing')|
	    (songname == 'playlock')|(songname == 'g1')|
	    (songname == 'g2')|(songname == 'g3')|
	    (songname == 'g4')|(songname == 'g5')|
	    (songname == 'g6')|(songname == 'g7')|
	    (songname == 'g8')|(songname == 'g9')|
	    (songname == 'g10')|(songname == 'playvalues')|
	    (songname == 'oldsong')|(songname == 'prevsong')|
	    (songname == 'currentsong')):
		ko = k
		k = db.nextkey(k)
		if k == ko:
		    break
		else:
		    continue
	    value = string.atoi(value)
	    if value == 1:
		one.append(songname)
	    elif value == 2:
		two.append(songname)
	    elif value == 3:
		three.append(songname)
	    elif value == 4:
		four.append(songname)
	    elif value == 5:
		five.append(songname)
	    elif value == 6:
		six.append(songname)
	    elif value == 7:
		seven.append(songname)
	    elif value == 8:
		eight.append(songname)
	    elif value == 9:
		nine.append(songname)
	    elif value == 10:
		ten.append(songname)
	    k = db.nextkey(k)
	songplayed = ' '
	oldplayfrom = playfrom   #save the old playfrom value so that
				 #playto can be reinitialized later
	whole_song_played = 0
	quit_or_not = end_thread(db)
	if quit_or_not == 1:
	    break
	#get the playvalues
	valuesfile = os.environ['HOME'] + "/.pimp3.values"
	f = open(valuesfile, 'r')
	p = pickle.Unpickler(f)
	playvalues = p.load()
	del f
	
	#this is ment to slightly speed up the random generator
	#it stops from generating numbers for depleted values
	count = 1
	while count <= 10:
	    key = count
	    if ((count == playfrom)&(playvalues[key] == 0)):
		playfrom = count + 1
	    if ((count == playto)&(playvalues[key] == 0)):
		playfrom = count - 1
	    count = count + 1
	    
	#generate the random number    
	if(playfrom != playto):           #accomodate the option to play only songs of the value 5
	    random = whrandom.randint(playfrom, playto)
	else:
	    random = playto
	
	#this handles the database and makes sure that 5 songs of the value
	#5, 4 of 4, and etc are played in each loop.
	    
	count = 1
	while count <= 10:
	    if ((count == random)&(playvalues[count] != 0)):
		songplayed = play(db, one, two, three, four, five, six, seven, eight, nine, ten, count, play_prev)
		value = playvalues[count]
		playvalues[count] = value - 1
	    count = count + 1
	    
	#reset values when nescessary
	count = oldplayfrom
	stop = 0
	while count <= 10:
	    if((playvalues[count] == 0) & (stop != 1)):
		stop = 0
	    else:
		stop = 1
	    count = count + 1
	    
	if stop != 1:
	    count = 1
	    playvalues = {}
	    while count <= 10:
		playvalues[count] = count
		count = count + 1
	    playfrom = oldplayfrom
	    playto = 5
		
	quit_or_not = end_thread(db)
	f = open(valuesfile, 'w')
	p = pickle.Pickler(f)
	p.dump(playvalues)
	del f
	if (quit_or_not == 0) & (songplayed != ' ') & (songplayed != None):
	    if(db.has_key(songplayed)):
		songvalue = db[songplayed]
	    else:
		print 'songname not in database'
		continue
	    print songvalue
	    del db[songplayed]
	    #print 'old value:', songvalue,
	    print 'songvalue upgraded by 1'
	    songvalue = database.addtostr(songvalue)
	    db[songplayed] = songvalue
	elif quit_or_not == 1:
	    break
	del one, two, three, four, five
    if(db.has_key('halt')):
	del db['halt']
	db['halt'] = 'b'
    else:
	db['halt'] = 'b'	
    if(db.has_key('playlock')):       #lock so that no other play functions can be started
	if db['playlock'] == 'locked':
	    del db['playlock']
	    db['playlock'] = 'unlocked'
    else:
	db['playlock'] = 'unlocked'
    thread.exit()
def play(db, one, two, three, four, five, six, seven, eight, nine, ten, value, play_prev):
    "play the assigned song"
    if value == 1:
	list = one
    elif value == 2:
	list = two
    elif value == 3:
	list = three
    elif value == 4:
	list = four
    elif value == 5:
	list = five
    elif value == 6:
	list = six
    elif value == 7:
	list = seven
    elif value == 8:
	list = eight
    elif value == 9:
	list = nine
    elif value == 10:
	list = ten
	
    length = len(list)
    if length > 0:
	random = whrandom.randint(0, length)
	random = random - 1
	songname = list[random]
	if play_prev == 1:
	    if(db.has_key('prevsong')):
		songname = db['prevsong']
	    else:
		db['prevsong'] = songname
		
	if(db.has_key('currentsong')):
	    del db['currentsong']
	    db['currentsong'] = songname
	else:
	    db['currentsong'] = songname
	filename = string.replace(songname, '\000', '')
	mp3name = '\"' + filename + '\"'
	command = "mpg123 " + mp3name
	filename = string.split(filename, '/') #strip off the directory heading
	length = len(filename)
	length = length - 1
	filename = filename[length]
	print filename, " : ", db[songname]
	print ":",
	if(db.has_key('oldsong')&db.has_key('prevsong')):
	    temp = db['oldsong']
	    del db['oldsong']
	    db['oldsong'] = songname
	    del db['prevsong']
	    db['prevsong'] = temp
	else:
	    db['oldsong'] = songname
	    db['prevsong'] = songname
	trash = commands.getstatusoutput(command)
	return songname
def play_help():
    print
    print 'help -this help message'
    print 'n    -play the next song'
    print 'v    -play the previous song'
    print '1-5  -change the current mp3\'s value to the number entered'
    print 'p    -start playing the music'
    print 'q    -quit'
    print
    
def randomplay(db, playfrom, versionnumber):
    "playsongs in random order"
    #used to make sure two instances of play are not running at the same time
    print 'Welcome to the pimp3 ',versionnumber,'!'
    if(db.has_key('playlock')):
	del db['playlock']   #remove the playlock
	db['playlock'] = 'unlocked'
    #start the menu loop
    play_help()
    #thread.start_new_thread(playsongs, (db, playfrom, 0))
    print ':',
    while 1:
	input = raw_input('')
	input = string.strip(input)
	if input == 'n':
	    next(db, playfrom)
	elif input == 'v':
	    prev(db, playfrom)
	elif input == 'p':
	    thread.start_new_thread(playsongs, (db, playfrom, 0))
	elif input == '1':
	    change_value(db, 1)
	elif input == '2':
	    change_value(db, 2)
	elif input == '3':
	    change_value(db, 3)
	elif input == '4':
	    change_value(db, 4)
	elif input == '5':
	    change_value(db, 5)
	elif input == 'q':
	    print 'Goodbye'
	    command = 'killall -9 mpg123'
	    try:
		del db['halt']
		db['halt'] = 'a'
	    except:
		db['halt'] = 'a'
	    commands.getstatusoutput(command)
	    thread.exit()
	    sys.exit()
	elif input == 'help':
	    play_help()
	else:
	    play_help()
	del input



















































import sys, string, commands, struct, os, gdbm, whrandom,database
datafmt = '100s'
def getlist(mp3folder):
    listcommand = 'find  ' + mp3folder + ' -name *.mp3'
    output = commands.getoutput(listcommand)
    output = string.split(output, '\012') #break apart files at the newlines
    return output
def int_to_string(value):
    if value <= 0:
	value = '0'
    elif value == 1:
	value = '1'
    elif value == 2:
	value = '2'
    elif value == 3:
	value = '3'
    elif value == 4:
	value = '4'
    elif value == 5:
	value = '5'
    elif value == 6:
	value = '6'
    elif value == 7:
	value = '7'
    elif value == 8:
	value = '8'
    elif value == 9:
	value = '9'
    elif value >= 10:
	value = '10'
    return value

def addtostr(value):
    value = string.atoi(value)
    value = value + 1
    value = int_to_string(value)
    return value
def subfromstr(value):
    value = string.atoi(value)
    value = value - 1
    value = int_to_string(value)
    return value
def rebuild(db, mp3folder, programname, notfirsttime, spreadratings):
    if(notfirsttime == 0):
	count = 0
	output = getlist(mp3folder)
	length = len(output)
	if spreadratings == 1:
	    one = 1
	    two = 2
	    three = 3
	    four = 4
	    five = 5
	    six = 6
	    seven = 7
	    eight = 8
	    nine = 9
	    ten = 10
	    
	    q = 1
	    while count < length: #distribute the ratings over 1-5
		q = whrandom.randint(1,10)
		if q == 1:
		    if one <= 0:
			q = q +1
			continue
		    else:
			one = one - 1
		elif q == 2:
		    if two <= 0:
			q = q + 1
			continue
		    else: 
			two = two - 1
		elif q == 3:
		    if three <= 0:
			q = q + 1
			continue
		    else:
			three = three - 1
		elif q == 4:
		    if four <= 0:
			q = q + 1
			continue
		    else:
			four = four - 1
			
		elif q == 5:
		    if five <= 0:
			q = q + 1
			continue
		    else:
			five = five - 1
			
		elif q == 6:
		    if six <= 0:
			q = q + 1
			continue
		    else:
			six = six - 1
			
		elif q == 7:
		    if seven <= 0:
			q = q + 1
			continue
		    else:
			seven = seven - 1
			
		elif q == 8:
		    if eight <= 0:
			q = q + 1
			continue
		    else:
			eight = eight - 1
		
		elif q == 9:
		    if nine <= 0:
			q = q + 1
			continue
		    else:
			nine = nine - 1
		elif q == 10:
		    if ten <= 0:
			q = q + 1
			continue
		    else:
			ten = ten - 1
		if (one + two + three + four + five + six + seven + eight + nine + ten<= 0):
		    one = 1
		    two = 2
		    three = 3
		    four = 4
		    five = 5
		    six = 6
		    seven = 7
		    eight = 8
		    nine = 9
		    ten = 10
		q = int_to_string(q)
		output[count] = string.strip(output[count])
		datapack = struct.pack(datafmt, output[count])
		db[datapack] = q
		count = count + 1
	else:
	    while count < length:
		output[count] = string.strip(output[count])
		datapack = struct.pack(datafmt, output[count]) 
		db[datapack] = '10'
		count = count + 1
    if(notfirsttime):
	count = 0
	databasefile2 = os.environ['HOME'] + "/." + programname + "-2.db"
        try:
            db2 = gdbm.open(databasefile2, 'w')
        except:
            db2 = gdbm.open(databasefile2, 'c')
            
	output = getlist(mp3folder)
	while count < (len(output) - 1):           #make a blank database with the songs in mp3folder
	    output[count] = string.strip(output[count])
	    datapack = struct.pack(datafmt, output[count])
	    db2[datapack] = '10'
	    count = count + 1
	k = db.firstkey()
	while k != None:           #save the current values
	    keyvalue = db[k]
	    if(db2.has_key(k)):
		del db2[k]
		db2[k] = keyvalue
	    k = db.nextkey(k)
	db = db2
	k = db.firstkey()
        while k != None:
            songname = struct.unpack(datafmt, k)     #delte the songs that no longer exist
	    songname = songname[0]
            filename = string.replace(songname, '\000', '')
            status = commands.getstatus(filename)
	    q = string.find(status, 'No such file')
	    if(q != -1):  #if the string doesn't exist
		del db[k]
	    k = db.nextkey(k)
def makesetup(db, mp3folder,programname):
    "print a setup file in the format mp3file : value"
    k = db.firstkey()
    setupfile = os.environ['HOME'] + "/" + programname + ".setup"
    f = open(setupfile, "w")
    while k != None:
	songvalue = db[k]
	songname = k
	songname = string.split(songname, '\000')
	songname = songname[0]
	songname = string.strip(songname)
	printstring = songname + " : "
	printstring = printstring + songvalue2
	printstring = printstring + '\n'
	f.write(printstring)
	k = db.nextkey(k)
	
def setup(db, programname):
    "take the setup file and put that info into the database"
    setupfile = os.environ['HOME'] + "/" + programname + ".setup"
    try:
	f = open(setupfile, "r")
    except:
	print "oops, you forgot to run imp3 -m first!"
	sys.exit()
    data = f.read()
    data = string.split(data, '\n')
    datamax = len(data)
    del data[datamax - 1]            #get rid of the empty line at the end of data
    count = 0
    while count < datamax - 1:
	i = string.split(data[count], ":")
	imax = len(i)
	imax = imax - 1
	if imax == 1:
	    stringdata = i[0]
	elif imax > 1:
	    q = 0
	    stringdata = ""
	    while q < imax:
		if q == 0:                      #this deals with mp3s with a : in their name
		    stringdata = stringdata + i[q]
		else:
		    stringdata = stringdata + ':' + i[q]
		q = q + 1
	stringdata = string.strip(stringdata)
	namedata = struct.pack(datafmt, stringdata)
	value = string.atoi(i[imax])
	value = int_to_string(value)
	try:
	    del db[namedata]
	except:
	    pass
	db[namedata] = value
	count = count + 1
